#!/usr/local/bin/perl -w
# (Normally, this script is called by install.sh, so the above line is not used)

# wginstall
# 
# Installation script for webglimpse v1.6 & above
# See included COPYRIGHT file for license info
# 

#| = 1;

$VERSION="1.6";

require "lib/wgserver.pl";

#####################################################################
# Welcome message
#
print "This is Webglimpse version $VERSION installation script\n";
print "For documentation see http://glimpse.cs.arizona.edu/webglimpse/install.html\n\n";

# Check that they extracted into a new directory

if (-e './.wgsiteconf') {
	print "Starting with Webglimpse 1.6, you should extract the installation files\n";
	print "into a temporary directory, NOT into the Webglimpse home directory\n";
	print "Don't worry, you can still upgrade cleanly, but you need to make a new directory\n";
	print "and extract the files there.  Then run wginstall from the new (temporary) directory.\n";
	exit;
}


# Get operating system
$LEGALOS = 'linux osf sunos solaris';
$DEFAULTOS = 'linux';
$PLATFORM = $ARGV[1] || $^O || '';

if ($PLATFORM eq '') {
	$PLATFORM = $DEFAULTOS;
	$_ = `uname -s` || '';
	tr/A-Z/a-z/;
	/^linux/ && ($PLATFORM = "linux");
	/^osf/ && ($PLATFORM = "osf");
	/^sunos/ && ($PLATFORM = "sunos");
	/^solaris/ && ($PLATFORM = "solaris");
}


do {
    $PLATFORM = prompt("What OS are you running (must be one of $LEGALOS)?", $PLATFORM);
} while ($LEGALOS !~ /$PLATFORM/);

###########################################################
# Make sure we can find needed system binaries
# Since not all systems have which, we make a guess if which fails
#
# get the paths for perl and glimpse
$PERL = $ARGV[0] || `which perl` || '/usr/local/bin/perl';
$CONVERT_LOC = `which wgconvert` || '/usr/local/bin/wgconvert';
$GLIMPSE_LOC = `which glimpse` || '/usr/local/bin/glimpse';
$GLIMPSEIDX_LOC = `which glimpseindex` || '/usr/local/bin/glimpseindex';
$GLIMPSE_VERSION = "4.0b1";
$gunzip = `which gunzip` || '/usr/local/bin/gunzip';
$CAT = `which cat` || '/bin/cat';

# Now see if the system binaries really exist where we think they are

# remove the '\n'
chomp $CAT;
if(!(-e $CAT)){
	$CAT="";
}
chomp $PERL;
if(!(-e $PERL)){
	$PERL="";
}
chomp $CONVERT_LOC;
if(!(-e $CONVERT_LOC)){
	$CONVERT_LOC="";
}
chomp $GLIMPSE_LOC;
if(!(-e $GLIMPSE_LOC)){
	$GLIMPSE_LOC="";
}
chomp $GLIMPSEIDX_LOC;
if(!(-e $GLIMPSEIDX_LOC)){
	$GLIMPSEIDX_LOC="";
}
chomp $gunzip;
if(!(-e $gunzip)){
	$gunzip="";
}
# 
##############################################################################
# Check with user if we cannot find system binaries
#
if($PERL eq ""){
	$PERL = &prompt("I cannot find perl!  Where is it?","");
}


if($GLIMPSE_LOC eq ""){
	print "\nWebglimpse requires Glimpse v4.0b1 or later.  The latest version of Glimpse\n"; 
	print "can be downloaded from http://glimpse.cs.arizona.edu\n\n";

	$GLIMPSE_LOC = &prompt("I cannot find glimpse!  Where is it?","");
}

# Try finding wgconvert in the same directory as glimpse
if ($CONVERT_LOC eq "") {
	$CONVERT_LOC = $GLIMPSE_LOC;
	$CONVERT_LOC =~ s/glimpse$/wgconvert/;
	if(!(-e $CONVERT_LOC)){
		$CONVERT_LOC="";
	}
}


if($CONVERT_LOC eq ""){
	$CONVERT_LOC = &prompt("I cannot find wgconvert! \n  It should have been installed with glimpse. \n   Where is it?","");
}


($ok, $GLIMPSE_VERSION) = &is_version_okay("$GLIMPSE_LOC -V", $GLIMPSE_VERSION);
if(! $ok){
	print "Webglimpse requires Glimpse v4.0b1 or later.  The latest version of Glimpse\n"; 
	print "can be downloaded from http://glimpse.cs.arizona.edu\n\n";

	$GLIMPSE_LOC = &prompt("I cannot find glimpse (version 4.0b1 or higher)!  Where is it?","");
} else {
	print "Found Glimpse v$GLIMPSE_VERSION. Good.\n";
}


if($GLIMPSEIDX_LOC eq ""){
	$GLIMPSEIDX_LOC = &prompt("I cannot find glimpseindex!  Where is it?","");
}

# if(!is_version_okay("$GLIMPSEIDX_LOC -V", $GLIMPSE_VERSION)){
# 	$GLIMPSEIDX_LOC = &prompt("I cannot find glimpseindex version 4.0b1 or higher)!  Where is it located?","");
# }

if($gunzip eq ""){
	$gunzip = &prompt("I cannot find gunzip using 'which'!  Where is it?","");
}

if($CAT eq ""){
	$CAT = &prompt("I cannot find cat using 'which'!  Where is it?","");
}
##############################################################################
# What can we do differently for Simple install?
# 
# Get user choice - simple or custom install?
#$INSTALLTYPE = prompt('Simple or Custom install? (S/c)', 'S');
#if ($INSTALLTYPE =~ /^[sS]/) {
#	$INSTALLTYPE = 'S';
#} else {
#	$INSTALLTYPE = 'C';
#}

######################################################################
# Get WEBGLIMPSE HOME DIRECTORY
# Try looking for confarc in the current path first
#
$guess = `which confarc` || '';
if ($guess ne '') {
	$guess =~ s/\/confarc$//;
}
chomp $guess;
if (($guess eq '')||($guess eq '.')) {
	$guess = '/usr/local/webglimpse';
}
$WEBGLIMPSE_HOME = prompt('Webglimpse Home Directory: ', $guess);

# Remove any trailing '/'
if ($WEBGLIMPSE_HOME =~ /\/$/) { 
	chop $WEBGLIMPSE_HOME;
}

$curdir = `pwd`;
if ($curdir =~ /\/$/) {
	chop $curdir;
}


# At this point, the user has chosen the same directory for WEBGLIMPSE_HOME as their current
# installation directory, but probably does NOT have an existing install in this directory
# because we already looked for .wgsiteconf earlier.
if (($curdir eq $WEBGLIMPSE_HOME)||($WEBGLIMPSE_HOME eq '.')) {
	print "\nStarting with Webglimpse 1.6, you should extract the installation files\n";
	print "into a temporary directory, NOT into the Webglimpse home directory\n";
	print "Please either choose another directory for Webglimpse Home,\n";
	print "or exit now and move the installation files elsewhere.\n\n";

	$WEBGLIMPSE_HOME = prompt('New Webglimpse Home Directory, return to exit: ', '');
	
	if ($WEBGLIMPSE_HOME eq '') {
		print "Ok, exiting wginstall.\n";
		exit;
	}	
}



if ( ! ( -d $WEBGLIMPSE_HOME ) ) {
	mkdir($WEBGLIMPSE_HOME, 0755) || die "\nERROR - Aborting install. Could not create directory $WEBGLIMPSE_HOME.\n  Error was $!\n";
}



# Check if .wgsiteconf already exists
$HAVECONFIG = 'N';
if ( -e "$WEBGLIMPSE_HOME/.wgsiteconf" ) {
	$HAVECONFIG = prompt('Existing installation found.  Keep configuration info? (Y/n)', 'Y');
	if ($HAVECONFIG =~ /^[yY]/) {
		$HAVECONFIG = 'Y';
	} else {
		$HAVECONFIG = 'N';
	}
}

#######################################################################
#
# Get configuration information if necessary
#
if ($HAVECONFIG eq 'N') {

	print "\nI will now ask you some questions about your HTTP server.\n";
	print "Automatic configuration is supported for Apache servers v1.2 and above\n";
	print "If you do not have an Apache server, you may need to edit the file\n";
	print "$WEBGLIMPSE_HOME/.wgsiteconf by hand after the installation is completed.\n\n";

	#		
	# Try to guess the configuration directory/file from the process list
	#
	$allprocs = `ps -a` || '';
	$confdir = '';
	$conffile = '';
	if ($allprocs ne '') {
		@allprocs = split(/\n/,$allprocs);
		foreach $proc ( @allprocs ) {
			if ($proc =~ /httpd/) {
				if ($proc =~ / -f (\S+)/) {
					$conffile = $1;
				}
				if ($proc =~ / -d (\S+)/) {
					$confdir = $1;
				}
				last;
			}
		}
	}
	$HTTPD_CONF = '';
	if ($conffile ne '') {
		# Usually the daemon is run as httpd -f /path/httpd.conf
		$HTTPD_CONF = $conffile;
	}

 	if ($confdir ne '') {
		if ($confdir =~ /\/$/) {
			chop $confdir;
		}
		if ($HTTPD_CONF eq '') {
			$HTTPD_CONF = 'httpd.conf';
		}
		$HTTPD_CONF = $confdir.'/'.$HTTPD_CONF;
	} 

	if ($HTTPD_CONF eq '') {
		$HTTPD_CONF = '/usr/local/etc/httpd/conf/httpd.conf';
	}

	# Check with the user if our guess is right.
	$HTTPD_CONF = &prompt('Full path to HTTP server config file: ',$HTTPD_CONF);

	########################################################################
	# Call wgSiteConfig from wgserver.pl to parse server config file
	# and write the .wgsiteconf file

	$mdocroot = wgSiteConfig($HTTPD_CONF,"$WEBGLIMPSE_HOME/.wgsiteconf", "");

	print "\nUsing Document Root of $mdocroot\n\n";
	print "Created configuration file $WEBGLIMPSE_HOME/.wgsiteconf\n";
	print "You can edit the contents of this file by hand if necessary\n";
	print "See $WEBGLIMPSE_HOME/example.wgsiteconf for a documented example.\n";
	print "\n";
}


######################################################################
# We need to know the cgi-bin directory
# to copy webglimpse to
#

# The only place we stored the real webglimpse cgi-bin directory in 
# versions 1.5 and earlier, was in the cgi-bin/webglimpse file (and a few of the dist files)
# and even then it was only the url path to it.

$CGIBIN = "cgi-bin";
if ( -e "$WEBGLIMPSE_HOME/cgi-bin/webglimpse" ) {

	if (open(F, "$WEBGLIMPSE_HOME/cgi-bin/webglimpse")) {
	
		while (<F>) {
			if (/\$CGIBIN = \"([^\/])\/webglimpse\"/) {
				$CGIBIN = $1;
				last;
			}
		}
		close(F);
	}
}

# Read default CGI-BIN info if available
# In version 2.0 and above, we have a line in .wgsiteconf
# WGSCRIPTALIAS urlpath abspath
# that holds the url and directory where webglimpse lives
#

$CGIBIN_PWD = "$WEBGLIMPSE_HOME/cgi-bin";
$wgbin = '';
$wgbin_pwd = '';

if (open(F, "$WEBGLIMPSE_HOME/.wgsiteconf")) {

	$found_cgi = 0;
	while (<F>) {

		# We prefer the 2.0 configuration if we can find it
		if (/^WGSCRIPTALIAS/) {
			($nullvar, $CGIBIN, $CGIBIN_PWD) = split(/\s/);
			chomp $CGIBIN;
			chomp $CGIBIN_PWD;
			$wgbin = $CGIBIN;
			$wgbin_pwd = $CGIBIN_PWD;
			last;
		}

		# If not, we'll take the first SCRIPTALIAS line
		if (/^SCRIPTALIAS/ && ($found_cgi == 0)) {
			($nullvar, $CGIBIN, $CGIBIN_PWD) = split(/\s/);
			chomp $CGIBIN;
			chomp $CGIBIN_PWD;
			$found_cgi = 1;
		}
	}
	close(F);
} 




# Prompt user no matter what; we may be overwriting a file here.
$CGIBIN_PWD = 
	&prompt("Which directory should I copy the WebGlimpse cgi-bin scripts to?\nIf you choose a different\ndirectory than the default, make sure it exists.\n",
	"$CGIBIN_PWD");

$CGIBIN = 
	&prompt("What is the script alias for $CGIBIN_PWD?",
		$CGIBIN);

# If this is news to us, add/change the WGSCRIPTALIAS line to the site config file
$wg_added = 0;
if (($CGIBIN_PWD ne $wgbin_pwd) || ($CGIBIN ne $wgbin)) {
	open(F, "$WEBGLIMPSE_HOME/.wgsiteconf");
	@wlines = <F>;
	close(F);
	if (open(F, ">$WEBGLIMPSE_HOME/.wgsiteconf")) {
		foreach $line (@wlines) {
			if ($line =~ /^WGSCRIPTALIAS/i) {
				print F "WGSCRIPTALIAS $CGIBIN $CGIBIN_PWD\n";
				$wg_added = 1;
			} else {
				print F $line;
			}
		}
		if (! $wg_added) {
			print F "WGSCRIPTALIAS $CGIBIN $CGIBIN_PWD\n";				
		}
		close(F);
	} else {
		print "Warning: Could not open $WEBGLIMPSE_HOME/.wgsiteconf for writing\n";
	}
}


# format the paths correctly
# there should be no trailing \n's
# NO trailing '/'s
if($CAT=~/\/$/) {
	chop $CAT;
}
if($PERL=~/\/$/) {
	chop $PERL;
}
if($CONVERT_LOC=~/\/$/) {
	chop $CONVERT_LOC;
}
if($GLIMPSE_LOC=~/\/$/) {
	chop $GLIMPSE_LOC;
}
if($GLIMPSEIDX_LOC=~/\/$/) {
	chop $GLIMPSEIDX_LOC;
}
if($gunzip=~/\/$/) {
	chop $gunzip;
}
if($WEBGLIMPSE_HOME=~/\/$/) {
	chop $WEBGLIMPSE_HOME;
}
if($CGIBIN_PWD=~/\/$/) {
	chop $CGIBIN_PWD;
}
if($CGIBIN=~/\/$/) {
	chop $CGIBIN;
}

# no beginning '/' on cgibin_url
if($CGIBIN=~/^\//) {
	$CGIBIN=substr($CGIBIN,1);
}
	

# prompt for correctness
print "-----------------------------------------------------\n";
print "perl: $PERL\n";
print "gunzip: $gunzip\n";
print "cat: $CAT\n";
print "\n";
print "glimpse: $GLIMPSE_LOC\n";
print "glimpseindex: $GLIMPSEIDX_LOC\n";
print "\n";
print "WebGlimpse Home Directory: $WEBGLIMPSE_HOME\n";
print "An existing cgi-bin directory: $CGIBIN_PWD\n";
print "cgi-bin *relative* URL: $CGIBIN\n";
print "-----------------------------------------------------\n";

print "To continue with these settings, press RETURN.  Otherwise, press ^C to abort.\n";
$_=<STDIN>;


# build local copy of httpget and html2txt
print "Trying to build httpget...\n";
unlink("lib/httpget");
unlink("lib/html2txt");
system("make $PLATFORM");


# if it worked, make the file executable by all
if(-e "lib/httpget" && -e "lib/html2txt"){
	print "BUILD SUCCESSFUL.\n";

}else{
	### TO DO -- update $HTTPGET_CMD
	# DO NOT use httpget.pl and url_get; they're too slow.
	# $GETHTTP_CMD = "\$WEBGLIMPSE_LIB/httpget.pl";
	print "BUILD FAILED.\n";
	print "Try to build httpget and html2txt later... \nWebGlimpse will not work well without them.\nSee Makefile for more details.\n";
	exit(0);
}
$GETHTTP_CMD = "\$WEBGLIMPSE_LIB/httpget";

# Files to copy
@cgibinfiles=("webglimpse", 
				"webglimpse-fullsearch",
#				"catarch",
				"mfs");



@glimpsebins=("getfile",
				"addsearch",
				"confarc",
				"rmarc",
				"makecron",
				"makenh",
				"Makefile");

@binarylibs = (				"httpget",
					"html2txt");

@libfiles = (
					"ftplib.pl",
					"url_get.pl",
					"httpget.pl",
					"utils.h",
					"URL.pl",
					"siteconf.pl",
					"wgserver.pl",	# New with v1.6
					"webgutils.pl",
					"OutputTool.pm", # New with v1.6b3
					"config.pl",
					"httpget.c",
					"url_get");

@optlibfiles = (			"CustomOutputTool.pm", # New with v1.6b4 - only distributed w/commercial version
					"ResultCache.pm"); # New with v1.6b5, only distributed with commercial version

@distfiles = (	
#					"archive.html",
					"glimpse-eye.jpg",
#					"glimpse_exclude",
					"glimpse_filters",
					"wgfilter-box",
					"wgfilter-index",
					"wgbox.html",
					"wgindex.html",
					"wgall.html");         # Added wgall 1/5/98 --GB

@optdistfiles = (			"wgoutput.cfg");  # Added customoutput.cfg 6/25/98 --GB


# We already created $WEBGLIMPSE_HOME if it did not exist
# Create the subdirectories dist, lib and CGIBIN_PWD if necessary
if ( ! ( -d "$WEBGLIMPSE_HOME/lib" ) ) {

	mkdir("$WEBGLIMPSE_HOME/lib", 0755) || die "ERROR - Aborting install. Cannot create directory $WEBGLIMPSE_HOME/lib\n";
}

if ( ! ( -d "$WEBGLIMPSE_HOME/dist" ) ) {

	mkdir("$WEBGLIMPSE_HOME/dist", 0755) || die "ERROR - Aborting install. Cannot create directory $WEBGLIMPSE_HOME/dist\n";
}

if ( ! ( -d $CGIBIN_PWD ) ) {

	mkdir("$CGIBIN_PWD", 0755) || die "ERROR - Aborting install. Cannot create directory $CGIBIN_PWD\n";
	print " Created $CGIBIN_PWD.  Please ensure that 
		\n\nScriptAlias $CGIBIN $CGIBIN_PWD
		\n\n is included in your HTTP daemon configuration files.\n";
}


# Copy all the files to $WEBGLIMPSE_HOME, overwriting anything except config files
# Change lines as we go.
foreach $file ( @glimpsebins ) {
	InstallFile($file, '', $WEBGLIMPSE_HOME, 0755, 0);
}

foreach $file ( @libfiles ) {
	InstallFile($file, '/lib', "$WEBGLIMPSE_HOME/lib", 0755, 0);
}
foreach $file ( @distfiles ) {
	InstallFile($file, '/dist', "$WEBGLIMPSE_HOME/dist", 0644, 0);
}
foreach $file ( @cgibinfiles ) {
	InstallFile($file, '/cgi-bin', $CGIBIN_PWD, 0755, 0);
}

# Now copy the optional modules & config files, if we have them
foreach $file ( @optlibfiles ) {
	InstallFile($file, '/lib', "$WEBGLIMPSE_HOME/lib", 0755, 1);
}
foreach $file ( @optdistfiles ) {
	InstallFile($file, '/dist', "$WEBGLIMPSE_HOME/dist", 0644, 1);
}

# Don't change lines in the binaries!
foreach $file ( @binarylibs ) {
	InstallBinaryFile($file, '/lib', "$WEBGLIMPSE_HOME/lib", 0755);
}

# create the dist/glimpse_filters file
eval{
	open(FILE, ">$WEBGLIMPSE_HOME/dist/glimpse_filters");
};
if($@){
	warn "Unable to create dist/glimpse_filters file!  Please create it manually!!\n";
}else{
	print FILE "*.html\t$WEBGLIMPSE_HOME/lib/html2txt <\n";
	print FILE "*.shtml\t$WEBGLIMPSE_HOME/lib/html2txt <\n";
	print FILE "*.sht\t$WEBGLIMPSE_HOME/lib/html2txt <\n";
	print FILE "*.htm\t$WEBGLIMPSE_HOME/lib/html2txt <\n";
	close FILE;
}


# create the archives file if doesn't exist
if (! -e "$WEBGLIMPSE_HOME/.archives") {

	if (open(FILE, ">$WEBGLIMPSE_HOME/.archives")) {
	
		print FILE "# Archive list for Webglimpse\n";
		print FILE "# ID#  Name          URL                     Hierarchy ID    Context ID  Description\n\n";
		close FILE;

#		# Now we make the archives file writable by the web server 
#		print "To use the web categorization interface, one configuration file must be made\n";
#		print "writable by the web user.  If you prefer not to do this, just hit return at the prompt.\n";
#		$wuser = &prompt("What username does the web server run as?", "");
#		if ($wuser ne '') {
#			system("chown $wuser $WEBGLIMPSE_HOME/.archives");
#		}		

	} else {
		print "WARNING: Unable to create $WEBGLIMPSE_HOME/.archives file.  \n";
	}
}


#### 
#### TODO: Check list in .archives and reconfigure each one with new version (prompt user 1st)
####

print "\n\n------------------------------------------------------------------------\nDONE with WebGlimpse installation.\n\n";

print "Webglimpse installation complete.\n\n";

print "**NOTE: You may need to edit your web server configuration file \n";
print "and specify $CGIBIN_PWD as a cgi-bin directory.\n";
print "For Apache, use the ScriptAlias directive.\n\n";
print("Press RETURN to continue\n");
<STDIN>;


print "\nSee http://glimpse.cs.arizona.edu/webglimpse for more\n";
print "information on WebGlimpse.\n\n";

print "You may configure an archive at any time with the command \n";
print "$WEBGLIMPSE_HOME/confarc\n\n";

print "To configure one or more archives now, just enter the directory to index or URL to traverse.\n";
print "A directory should be a full path starting with   /  \n";
print "A URL should be absolute starting with    http://  \n\n";

$archivepath = &prompt("Enter directory or URL to index, return to quit:", '');
while($archivepath ne '') {

	# We want to make certain simplified assumptions here.
	# User can get full flexibility by running confarc from
	# command line later.
	$type = 'D';
	if ($archivepath =~ /^http/) {
		$type = 'T';
	}

	system("$WEBGLIMPSE_HOME/confarc -i $archivepath -t $type -l $archivepath");

	$archivepath = &prompt("Enter directory or URL to index, return to quit:", '');
}

1;



##############################################################################
## Subroutines
##############################################################################
sub InstallBinaryFile() {
	local($filename, $srcdir, $targetdir, $perms) = @_;

	my $file = ".$srcdir/$filename";
	my $targetfile = "$targetdir/$filename";

	if (system("cp $file $targetfile") !=0 ){
		die "ERROR - Aborting install. Cannot copy $file to $targetfile.\n";
	}

	# set permissions of target file
	if(chmod ($perms, $targetfile)==0) {
		warn "WARNING. Cannot change permissions of $targetfile to $perms.\n";
	}

	return 1;
}


sub InstallFile() {

	local($filename, $srcdir, $targetdir, $perms, $opt_flag) = @_;

	my $tempfile = './tmpfile';
	my $file = ".$srcdir/$filename";
	my $targetfile = "$targetdir/$filename";


	# Check if optional files exist
	$opt_flag && (! -e $file) && return;

	# open the file
	open(FILE, $file) || die "ERROR - Aborting install.   Cannot open file $file: ";

	# open a temp file 
	open(TEMP, ">$tempfile") 
		||die "ERROR - Aborting install.   Cannot open temporary file: ";
	
	while(<FILE>){
		if(/^\$PERL *=/){
			print TEMP "\$PERL = \"$PERL\";\n";
		}elsif(/^\$GETHTTP_CMD *=/){
			print TEMP "\$GETHTTP_CMD = \"$GETHTTP_CMD\";\n";
		}elsif(/^\$CAT *=/){
			print TEMP "\$CAT = \"$CAT\";\n";
		}elsif(/^\$CONVERT_LOC *=/){
			print TEMP "\$CONVERT_LOC = \"$CONVERT_LOC\";\n";
		}elsif(/^\$GLIMPSE_LOC *=/){
			print TEMP "\$GLIMPSE_LOC = \"$GLIMPSE_LOC\";\n";
		}elsif(/^\$GLIMPSEIDX_LOC *=/){
			print TEMP "\$GLIMPSEIDX_LOC = \"$GLIMPSEIDX_LOC\";\n";
		}elsif(/^\$WEBGLIMPSE_HOME *=/){
			print TEMP "\$WEBGLIMPSE_HOME = \"$WEBGLIMPSE_HOME\";\n";
		}elsif(/^\$CGIBIN *=/){
			print TEMP "\$CGIBIN = \"$CGIBIN\";\n";
		}elsif(/^\$gunzip *=/){
			print TEMP "\$gunzip = \"$gunzip\";\n";
		}elsif(/^#!\/usr\/local\/bin\/perl$/){
			print TEMP "#!$PERL\n";
		}else{
			print TEMP $_;
		}
	}
	close (TEMP);
	close (FILE);

	# We need to copy instead of rename, because might be across filesystems
	# overwrite $targetfile with $tempfile
	if (system("cp $tempfile $targetfile") !=0 ){
		die "ERROR - Aborting install. Cannot copy $tempfile to $targetfile.\n";
	}
	system("rm $tempfile");

	# set permissions of target file
	if(chmod ($perms, $targetfile)==0){
		warn "WARNING. Cannot change permissions of $targetfile to $perms.\n";
	}

}




sub is_version_okay {
	my($cmd, $ver) = @_;
	my(@output) = `$cmd`;
	my($num, $thisnum);
	my(@line) = split(/\s/,$output[1]);
	$thisver = $line[4];  # "This is xxx version x.x"

	$thisnum = $thisver;
	$thisnum =~ s/[^0-9.]//g;
	$num = $ver;
	$num =~ s/[^0-9.]//g;

	if($thisnum >= $num){
		$ans = 1;
	}else{
		$ans = 0;
	}
	return $ans,$thisver;
}


##########################################################################
sub prompt {
	local($prompt,$def) = @_;
	print $prompt,"[",$def,"]:";
	$| = 1;
	$_ = <STDIN>;
	chomp;
	return $_?$_:$def;
}

1;


